#ifndef _WLAN_API_H_
#define _WLAN_API_H_

#include <NetMgr.h>


//----------------------------------------------------------------------------
// API version
//----------------------------------------------------------------------------
// 1.0      Initial version in first release driver (1.0.0)
// 1.1      Change definition of setting WPA-PSK Passphrase
// 1.2      Add some private API calls for testing.
// 1.3      Implement WPA passphrase API. Remove internal Pref IDs. Add 
//          roaming list management APIs. Allow read of certain security 
//          settings, formerly read-only.
//----------------------------------------------------------------------------
#define wlanAPIVersionMajor         1
#define wlanAPIVersionMinor         3


#define wlanNetIFDbName             "WLAN NetIF"
#define wlanNetIFServiceName        "WLAN"          // should match NPPI
#define wlanNetIFCreator		    'BGWb'


/*---------------------------------------------------------------------------
 * Some of these may be returned through API, but most are internal use only
 *---------------------------------------------------------------------------*/
#define wlanErrorBase                  0x8000
#define wlanErrAdapterFailed		   (wlanErrorBase | 0)	// some adapter problem
#define wlanErrNotResponding		   (wlanErrorBase | 1)  // adapter not responding
#define wlanErrSetupFailed             (wlanErrorBase | 2)  // error setting up ethernet controller
#define wlanErrDHCPTimeout             (wlanErrorBase | 3)  // no DHCP replies
#define wlanErrUserCancel              (wlanErrorBase | 4)  // user canceled connect
#define wlanErrAssociationTimeout      (wlanErrorBase | 5)  // associate timeout
#define wlanErrRxBusy      			   (wlanErrorBase | 6)  // could not process RX packet
#define wlanErrTxBusy      			   (wlanErrorBase | 7)  // could not process TX packet
#define wlanErrSettingNotAvailable     (wlanErrorBase | 8)  // NetIF setting not available
#define wlanErrScanTimeout             (wlanErrorBase | 9)  // a scan timeout occurred
#define wlanErrPrefsReinitialized      (wlanErrorBase | 10) // preferences were reset to defaults
#define wlanErrNotAssociated           (wlanErrorBase | 11) // wlan not current connected
#define wlanErrWriteOnlySetting        (wlanErrorBase | 12) // setting is not readable
#define wlanErrUsingDefaultMacAddr     (wlanErrorBase | 13) // could not read a valid MAC address from flash or EEPROM
#define wlanErrEEPROMAccessFailed      (wlanErrorBase | 14) // EEPROM could not be accessed
#define wlanErrEEPROMInvalid           (wlanErrorBase | 15) // EEPROM data is invalid
#define wlanErrEEPROMLocked            (wlanErrorBase | 16) // EEPROM is locked against writing
#define wlanErrEmptyPrefSSIDList       (wlanErrorBase | 17) // Preferred SSID list is empty
#define wlanErrPrefSSIDNotFound        (wlanErrorBase | 18) // Preferred SSID not found in list

/*----------------------------------------------------------------------------------------------------
 *                          NetIF settings for the WLAN network interface. 
 *
 * See the Network Library section of the Palm OS API Reference for details on using the 
 * NetLibIFSettingGet and NetLibIFSettingSet calls. What follows is a description of the WiFi-
 * specific calls implemented by this driver. The driver also implements the standard settings 
 * that are applicable to Ethernet interfaces, such as netIFSettingUp, netIFSettingReqIPAddr, etc.
 *
 * This driver also supports the legacy netIFSettingS24... settings defined by the Symbol S24
 * WLAN API for Palm OS 4.0, though these calls are for backwards compatibility only and should
 * not be used in new software development. Existing software using the Symbol API can continue
 * to include the S24API.h header file, but must redfine the driver creator code netIFCreatorS24
 * to match wlanNetIFCreator (ie. change 's24i' to 'BGWb'). For new applications, the API defined
 * in this header file, along with the standard API provided by Palm OS 5.x, duplicates all the 
 * functionality of the Symbol API except for the "set" function of netIFSettingS24EssID (see
 * description of netIFSetting80211ESSID for details) and and ShowESSID preference. To disable
 * DNS, the caller should use NetLibSettingSet to zero out the two DNS server entries; if DHCP
 * is used, make sure to also set netIFSetting80211ManualDNS to 1. Note: for drivers supporting
 * roaming (1.2.x and above) changing the SSID on the fly using netIFSettingS23EssID is not 
 * supported, and may cause unpredictable behavior (on driver 1.2.1 and above this call is disabled).
 * 
 * Versions 1.0.x and 1.1.x of this driver support only standard WEP security (both 40/64-bit and 
 * 104/128-bit), while version 1.2.x adds support for WPA-PSK. Future versions may add WPA-Enterprise 
 * support (ie. EAP methods and RADIUS support). Use the APIs netIFSetting80211HighestSecurityMode 
 * and netIFSetting80211HighestEncryptionType to determine the driver security support at runtime. 
 * New API settings may be added in the future to support additional security/encryption options, 
 * as needed.
 *
 * API call descriptions are grouped according to the driver version on which they first appeared.
 * See section below for APIs and features added in driver 1.2.x.
 * 
 *
 *-----------------------------------------------------------------------------------------------------
 *                      Driver version 1.0.x and 1.1.x (API versions 1.0, 1.1, 1.2)
 *
 * This is the basic WEP-only driver. It supports WEP64 and WEP128. This driver does not roam, though
 * an application could implement rudimentory roaming using the Scan and RSSI settings to collect 
 * access point information, and the Symbol S24EssID setting to switch access points on the fly. Note,
 * however, that Palm OS will immediately bring down the driver if the association is completely lost.
 *
 * The current connection parameters are defined by a set of internal preferences now termed "legacy"
 * settings (as of the new 1.3 API). These settings define a single SSID, and the 1.0.x/1.1.x driver 
 * always connects to this SSID using these settings when it is brought up. Multiple SSID configurations
 * are stored locally in the WLAN Network Preference Panel, and the selected SSID configuration is
 * written to the driver when the user selects the SSID from the scan list in the panel.
 *
 * This driver supports open connections and WEP security. There are two different types of WEP, based
 * on the key length. The 40-bit WEP keys (also called 64-bit keys) consist of 10 hex digits or 5 ASCII
 * characters. The 104-bit WEP keys (also called 128-bit keys) consist of 26 hex digits or 13 ASCII
 * characters. WEP key entry via passphrase is not supported in this version.
 *
 *-----------------------------------------------------------------------------------------------------
 * 
 * netIFSetting80211...  Datatype  Description
 * -----------------     --------  --------------------------------------------------------------------
 *  Device               UInt8      Read-only. Can be read at any time. Returns 1, indicating an 802.11 
 *                                  interface. This is a standard Palm OS setting.
 *
 *  ESSID                Char[32]   Read/write. Can be read/written at any time. When the interface is
 *                                  down, read returns the ESSID preference, to be used for the next 
 *                                  connection. When the interface is up, read returns the current ESSID 
 *                                  of the connection. Writing always writes the preference. ESSID is 
 *                                  printable but not assumed to be NULL-terminated. This is a standard 
 *                                  Palm OS setting. As of API 1.3, this is considered a legacy API
 *                                  and is replaced by the Preferred SSID List. 
 *
 *                                  The Symbol S24 provides a slightly different version of this setting,
 *                                  netIFSettingS24EssID. The get functionality is the same, but calling
 *                                  NetLibIFSettingSet with Symbol's netIFSettingS24EssID while the 
 *                                  interface is connected will cause it to attempt to associate with the
 *                                  specified access point. Note that for this to work, the current and
 *                                  new access points must be on the same network (since IP address are
 *                                  not reassigned) and all security settings must match. Using 
 *                                  netIFSettingS24EssID to switch access points is not recommended, due
 *                                  to possible interference on future versions of the driver that 
 *                                  support roaming.
 *
 *  AccessPointBSSID     UInt8[6]   Read-only. Can only be read while driver is up. Returns the BSSID of 
 *                                  the current connection (usually the MAC address of the access point). 
 *                                  This is a standard Palm OS setting.
 *
 *  AssociationStatus    UInt8      Read-only. Can be read at any time. Returns 1 if the interface is 
 *                                  associated, 0 if it is not. Normally, the interface will always be 
 *                                  associated when the interface is up, and the loss of the association 
 *                                  will cause the interface to come down, though this behavior may change 
 *                                  in future driver versions that support roaming. Also, the driver is
 *                                  not associated when brought up in "scan" mode (see the setting
 *                                  netIFSetting80211ScanMode, below). This is a standard Palm OS setting.
 *
 *  MKKCallSign          Char[15]   Read-only. Can be read at any time. Currently not supported, and 
 *                                  always returns an empty string. This is a standard Palm OS setting.
 *
 *  CountryText          Char[32]   Read-only. Can be read at any time. Returns a NULL-terminated country 
 *                                  name. This is actually derived from the prefLocale setting on the device. 
 *                                  The WiFi driver uses the Locale setting to determine the allowed 802.11 
 *                                  channel scan list and maximum transmit power. This is a standard Palm 
 *                                  OS setting.
 *
 *  APIVersion           UInt16     Read-only. Can be read at any time. Returns the API major version in
 *                                  the most-significant byte and the API minor version in the least 
 *                                  significant byte. The API version is also defined at the top of the
 *                                  WLAN_API.h header file. API changes made after version 1.0 will be
 *                                  commented as such.
 *
 *  ReqESSID             Char[32]   Read/write. Can be read/written at any time. Accesses the preferred
 *                                  ESSID stored in the driver preferences. This is the ESSID that will be 
 *                                  used for the next connection. For NetLibIFSettingSet, this setting 
 *                                  behaves identically to netIFSetting80211ESSID. For NetLibIFSettingGet, 
 *                                  this setting always returns the value in the preferences, whereas 
 *                                  netIFSetting80211ESSID returns the value from the preferences only if 
 *                                  the interface is down. The ESSID is is printable but not assumed to be 
 *                                  NULL-terminated.
 *
 *  RSSI                 Int16      Read-only. Can only be read while driver is up. Returns the received 
 *                                  signal strength of the current connection. The value is in dBm and 
 *                                  ranges from 0 to -200, with increasing negative numbers indicating
 *                                  decreasing signal strength.
 *          
 *  NetworkType          UInt16     Read/write. Can be read/written at any time. Allowed values are
 *                                  _80211_NETWORK_TYPE_INFRUSTRUCTURE or _80211_NETWORK_TYPE_AD_HOC. When
 *                                  writing, sets the network type preference for the next connection. 
 *                                  When reading, returns the current network type if connected, otherwise 
 *                                  returns the network type from the driver preferences. As of API 1.3,
 *                                  this is considered a legacy API and is replaced by the Preferred
 *                                  SSID List.
 *
 *  TxRate               UInt16     Read-only. Can only be read while driver is up. Returns the current 
 *                                  transmit data rate in Mb/sec. Expected values are 1, 2, 5, or 11. A 
 *                                  value of 5 corresponds to 5.5 Mbps.
 *                        
 *  Channel              UInt16     Read-only. Can only be read while driver is up. Returns the channel 
 *                                  used for the current connection. 
 * 
 *  ReqAdHocChannel      UInt16     Read/write. Can be read/written at any time. Get/set the channel 
 *                                  preference for ad-hoc (peer-to-peer) network connections. Allowed
 *                                  range of values is determined by the current device locale setting.
 *                                  As of API 1.3, this is considered a legacy API and is replaced by
 *                                  the Preferred SSID List.
 *
 *  MACAddress           UInt8[6]   Read-only. Can be read only while driver is up. Returns the hardware 
 *                                  MAC address of the adapter. This is read from a pre-programmed EEPROM 
 *                                  on the WLAN adapter unless overridden in the driver preferences.
 *
 *  ReqMACAddr           UInt8[6]   Read/write. Can be read/written at any time. Get/set the preferred 
 *                                  MAC address, used to override the pre-programmed MAC address in the 
 *                                  WLAN adapter. To delete a MAC address from the preferences, set to a 
 *                                  value of length 0.
 *
 *  ManualDNS            UInt8      Read/write. Can be read/written at any time. Get/set manual DNS mode.
 *                                  Only used when DHCP enabled. When using DHCP, setting ManualDNS to 1
 *                                  allows the application to set DNS addresses manually through NetLib,
 *                                  ignoring the DNS addresses acquired from the DHCP response. Setting
 *                                  ManualDNS to 0 (default) causes NetLib to use the DNS addresses from
 *                                  the DHCP response. As of API 1.3, this is considered a legacy API
 *                                  and is replaced by the Preferred SSID List.
 *
 *  AuthenticationMode   UInt16     Read/write. Can be read/written at any time. Get/set the authentication 
 *                                  mode to be used in the next connection, stored in the driver
 *                                  preferences. This is the basic 802.11 authentication step performed 
 *                                  before association, not a WPA authentifaction method. The possible 
 *                                  values are _80211_AUTH_MODE_OPEN and _80211_AUTH_MODE_SHARED_KEY. 
 *                                  As of API 1.3, this is considered a legacy API and is replaced by
 *                                  the Preferred SSID List.
 *
 *  Statistics           Stats_t    Read-only. Can be read only while the driver is up. Returns a 
 *                                  structure containing the latest packet statistics. These are 
 *                                  seperate from the statistics maintained by NetLib, but contain
 *                                  similar information.
 *
 *  Scan             AP_ScanInfo_t  Read-only. Can be read only while the driver is up. Causes the
 *                                  driver to perform a scan, and then returns the results. If the scan
 *                                  times out (longer than 10 seconds) wlanErrScanTimeout is returned.
 *                                  This is the blocking version of scan. For non-blocking scans (the
 *                                  preferred method) see netIFSetting80211StartBackgroundScan, below.
 *                                  Upon successful scan, this function returns a list of networks
 *                                  of type AP_ScanInfo_t. The first field, NumScanResults, gives the
 *                                  number of networks found (up to 12 total). The second field is an
 *                                  array of AP_ScanResult_t elements, each describing one network.
 *                                  The useful fields of AP_ScanResult_t are:
 *                                         
 *                                          MacAddress: BSSID of the network (for an access point,
 *                                                      the MAC address of the access point).
 *                                          SSIDLength: Number of characters in the SSID.
 *                                          SSID:       SSID of the network. Generally printable
 *                                                      but not NULL terminated; use SSIDLength to
 *                                                      determine length of network name.
 *                                          BSSType:    Type of network (infrustructure or ad-hoc).
 *                                                      See netIFSetting80211NetworkType for values.
 *                                          RSSI:       Signal strength in dBm. This should be
 *                                                      interpreted as an Int32, and has the same
 *                                                      range as netIFSetting80211RSSI.
 *                                          Capability: If bit 4 is set, network uses security,
 *                                                      otherwise it is open.
 *                              
 *                                  Other fields a used internally but no documented here.
 *
 *                                  In order to perform a scan, the driver must be loaded (up). To
 *                                  bring the driver up without connecting to a network, the caller
 *                                  must first set the "ScanMode" preference (see the setting
 *                                  netIFSetting80211ScanMode, below). If ScanMode is not set, the 
 *                                  driver must be up and connected to a network before a scan can
 *                                  be performed.
 *
 *                                  The driver interprets the first two bytes of the incoming buffer
 *                                  as a UInt16 containing an "extra parameters" length. These 
 *                                  parameters are used internally and not documented for applications.
 *                                  for this reason, the caller must zero out the first two bytes of
 *                                  the incoming buffer. For simplicity, simply set NumScanResults to
 *                                  0 before calling Scan.                      
 *
 *  StartBackgroundScan  <none>     Write-only. Can be written only while driver is up. Tells the 
 *                                  driver to start a scan but does not wait for scan to complete.
 *                                  Use netIFSetting80211ScanInProgress to check status of scan,
 *                                  and netIFSetting80211ScanResults to get results. This setting
 *                                  takes no parameters; the caller should pass in a buffer length
 *                                  of 0 to avoid unexpected behavior. See netIFSetting80211Scan
 *                                  for general details on scanning. Returns success if the scan
 *                                  is successfully started or if a scan is already in progress.
 *
 *  ScanInProgress       UInt8      Read-only. Can be read only while driver is up. Returns 1 if a
 *                                  scan is currently in progress, 0 otherwise. After starting a 
 *                                  background scan with netIFSetting80211StartBackgroundScan, read
 *                                  this setting to determine when the scan is complete. The 
 *                                  typical timeout for a scan is 10 seconds, however the driver
 *                                  does not time application-initiated scans, so it is up to the
 *                                  caller to decide if the scan has timed out. If a time-out is
 *                                  suspected, the caller must clear the scan status using 
 *                                  netIFSetting80211CancelScan before another scan can be started. 
 *                                  In practice, however, a scan should always complete, and a timeout 
 *                                  probably indicates a more serious driver/adapter failure; the
 *                                  best solution may be to bring the interface down and then back
 *                                  up again.
 *
 *  ScanResults      AP_ScanInfo_t  Read-only. Can be read only while driver is up. Returns the
 *                                  results of the last scan. See netIFSetting80211Scan for more
 *                                  information on scanning and the AP_ScanInfo_t structure.
 *
 *  CancelScan           <none>     Write-only. Can be written only while the driver is up. Used
 *                                  to reset the scan status if an application-initiated 
 *                                  background scan times out (as determined by the application). 
 *                                  See setting netIFSetting80211ScanInProgress for more details.
 *       
 *  ScanMode             UInt8      Read/write. Can be read/written at any time. Get/set the driver 
 *                                  ScanMode preference. If set to 1, the driver will not attempt
 *                                  to associate when brought up by NetLib. If set to 0, driver 
 *                                  will always associate when brought up by NetLib, and will not
 *                                  come up if the association fails. ScanMode is used to bring 
 *                                  the driver up for the purposes of choosing a network, and is
 *                                  used by the WLAN Network Preference Panel to display a list
 *                                  of available networks to the user. When the driver is up in
 *                                  ScanMode, no data can be send or received, so this value must
 *                                  be cleared to 0 before a data connection can be established.
 *                                  A typical sequence of events might be: 
 *
 *                                          (1) Set ScanMode to 1,
 *                                          (2) Bring up driver (ie. open NetLib)
 *                                          (3) Perform scan
 *                                          (4) Choose SSID from scan list
 *                                          (5) Bring down driver (ie. close NetLib)
 *                                          (6) Set netIFSetting80211ReqESSID to desired SSID
 *                                          (7) Set other required connection parameters
 *                                          (8) Set ScanMode to 0.
 *                                          (9) Bring up driver, which will try to connect to SSID
 *
 *  Roaming              UInt8      Read/write. Can be read/written at any time. Controls the roaming
 *                                  preference on drivers supporting roaming. Set to 1 to enable
 *                                  roaming on the next connection, set to 0 to disable roaming. If
 *                                  the driver does not support roaming, setting to 1 will return
 *                                  netErrParamErr. Driver version 1.0.x does not support roaming.
 *
 *  HighestSecurityMode  UInt16     Read-only. Can be read at any time. Returns the "highest"
 *                                  security mode supported by the driver. Possible values are:
 *
 *                                      _80211_SECURITY_NONE    no security support (won't see this)
 *                                      _80211_SECURITY_WEP     WEP-only security (driver release 1.0)
 *                                      _80211_SECURITY_PSK     WPA-Personal (WPA-PSK and WEP) support
 *                                      _80211_SECURITY_EAP     WPA-Enterprise support (EAP methods)
 *                                  
 *                                  Driver supports all modes up to the returned mode (ie. support
 *                                  for _80211_SECURITY_EAP implies all other modes are supported).
 *                                  Driver version 1.0.x supports only WEP security.
 *
 *  SecurityMode         UInt16     Read/write. Can be read/written at any time. Get/set the preferred
 *                                  security mode. Use netIFSetting80211HighestSecurityMode to learn
 *                                  which modes the driver supports. Set _80211_SECURITY_NONE for
 *                                  open networks. Note that all security modes other than NONE 
 *                                  require additional parameters to be setup, such as keys, etc.
 *                                  All security modes other than NONE require an encrption type
 *                                  to be set (see netIFSetting80211EncryptionType, below). Driver
 *                                  version 1.0.x supports only WEP security. This setting was
 *                                  read/only in API versions before 1.3. As of API 1.3, this is 
 *                                  considered a legacy API and is replaced by the Preferred SSID List.
 *
 *  HighestEncryptionType  UInt16   Read-only. Can be read at any time. Returns the "highest"
 *                                  encryption type supported. Possible values are:
 *
 *                                      _80211_ENCRYPT_WEP      WEP encryption engine
 *                                      _80211_ENCRYPT_TKIP     WPA TKIP encryption engine
 *                                      _80211_ENCRYPT_AES      802.11i AES encryption engine
 *                                    
 *                                  Driver supports all types up to the returned type (ie. support
 *                                  for _80211_ENCRYPT_AES implies all other types are supported).
 *                                  Driver version 1.0.x supports only WEP encryption.
 *  
 *  EncryptionType       UInt16     Read/write. Can be read/written at any time. Get/set the preferred
 *                                  encryption engine for a secure connection. Setting ignored for
 *                                  open connections (_80211_SECURITY_NONE). Not all combinations
 *                                  are valid. For example, WEP security mode requires WEP
 *                                  encryption. To avoid problems, set the security mode first,
 *                                  followed by encryption type. If incompatible, the error
 *                                  will be returned when the encryption type is set and a 
 *                                  compatible encryption type will automatically be chosen.
 *                                  Driver version 1.0.x supports only WEP encryption. This setting
 *                                  was read-only on API versions before 1.3. As of API 1.3, this is 
 *                                  considered a legacy API and is replaced by the Preferred SSID List.
 *  
 *  WEPKeyIndex          UInt16     Read/write. Can be read/written at any time. Get/set the index of 
 *                                  the active WEP key. Used for WEP encryption. Values are 0 through 
 *                                  3. This setting was read-only on API versions before 1.3. As of API 
 *                                  1.3, this is considered a legacy API and is replaced by the 
 *                                  Preferred SSID List.
 * 
 *  WEPKeyLength         UInt16     Read/write. Can be read/written at any time. Get/set the type of 
 *                                  WEP encryption, either 48 (64) or 104 (128) bit WEP. These settings
 *                                  equate to the length of the WEP keys, 48/104 indicating the 
 *                                  two actual key lengths entered by the user, and 64/128 indicating
 *                                  the expanded lengths used internally. Possible values are:
 *
 *                                      _80211_WEP_KEY_LENGTH_40_BIT   40-bit (10 character) WEP key
 *                                      _80211_WEP_KEY_LENGTH_64_BIT   synonym for 40-bit WEP
 *                                      _80211_WEP_KEY_LENGTH_104_BIT  104-bit (26 character) WEP key
 *                                      _80211_WEP_KEY_LENGTH_128_BIT  synonym for 40-bit WEP
 *
 *                                  When a key is set, the driver will check that the key-length
 *                                  matches the currently set WEPKeyLength. Therefore, this setting
 *                                  must be set before individual keys are set. All keys must be
 *                                  of the same length. This setting was read-only on API versions
 *                                  before 1.3. As of API 1.3, this is considered a legacy API and is 
 *                                  replaced by the Preferred SSID List.
 *
 *  WEPKey               WEPKey_t   Write-only. Can be written at any time.  Set a WEP key for WEP
 *                                  mode encryption. Structure contains three fields:
 *
 *                                      keyIndex:   0 - 3, index of key being set
 *                                      keyLength:  length of key, must match WEPKeyLength setting
 *                                      keyData:    5 (40-bit) or 13 (104-bit) bytes of key data
 *
 *                                  Setting netIFSetting80211KeyIndex chooses the actual key to
 *                                  be used for transmission. Field keyLength indicates the length
 *                                  of this key, but must match the global setting 
 *                                  netIFSetting80211WEPKeyLength or the key will be rejected. Key
 *                                  data is in binary; ASCII-based keys must be translated to
 *                                  binary by the caller. As of API 1.3, this is considered a legacy 
 *                                  API and is replaced by the Preferred SSID List.
 * 
 *  Passphrase           Char[64]   Write-only. Can be written at any time. Set the key/passphrase
 *                                  for WPA-PSK (pre-shared key) modes. Passphrase is a NULL-
 *                                  terminated ASCII string from 8 to 63 characters long or a binary
 *                                  key 32 bytes long. If the first byte of the data is 0, the 
 *                                  next 32 bytes are interpreted as a binary key, otherwise
 *                                  the data is interpreted as an ASCII passphrase. 
 *                          
 *                                  Although defined since API version 1.0, this setting was not
 *                                  was not implemented in the driver until version 1.2.x; older
 *                                  drivers will return netErrUnknownSetting. As of API 1.3, this 
 *                                  is considered a legacy API and is replaced by the Preferred SSID 
 *                                  List.
 *  
 *-----------------------------------------------------------------------------------------------------
 *                                 Driver version 1.2.x (API 1.3)
 * 
 * Driver version 1.2.x and above contain a new component known a the "Security Supplicant." This
 * is a seperate thread that takes control of the driver connection process, adding support for 
 * roaming and advanced (WPA/WPA2/802.11i) security modes. The supplicant code was ported from the
 * Open Source Linux WPA/WPA2/IEEE 802.1X Supplicant (wpa_supplicant) written by Jouni Malinen 
 * (http://hostap.epitest.fi/wpa_supplicant) and is licensed under the BSD license. See copy of 
 * license at bottom of this file.
 *
 *
 *      WPA/WPA2 security
 *
 * The purpose of the Security Supplicant is to manage the authentication and keying step of the
 * connection. For access points using open access or WEP encryption, this step is trivial and 
 * consists merely of setting some driver parameters. However, for WPA/WPA2/802.11i security, the
 * supplicant must communicate with the access point before the driver can complete the connection.
 * In general, three steps are involved. (1) The driver will associate with the access point, just 
 * as in open/WEP connections. (2) The security supplicant must authenticate the device to the 
 * authentication server and (optionally) authenticate the authentication server to the device. The
 * authentication server might be the access point or an external server on the local network. (3) 
 * The supplicant performs a key exchange with the access point to setup the encryption. 
 *
 * Version 1.2.x of the driver supports WPA "personal" security only. WPA personal security allows
 * three connection modes: open, WEP, and PSK. Open and WEP modes work as on the WEP-only driver. 
 * PSK mode uses a "pre-shared secret" to authenticate the device to the authentication server (in 
 * this case always the access point). The "pre-shared secret" or "pre-shared key" (PSK) may be 
 * either a 256-bit binary key or, more commonly, an ASCII passphrase from which the 256-bit key is 
 * internally generated by a standard hashing algorithm. Step (2) is thus eliminated and only step
 * (3) is performed by the supplicant.
 *
 * WPA security supports two encryption types: WEP (with two different key lengths) and TKIP. 
 * WPA2/802.11i support adds AES encryption. WEP mode always uses WEP encryption but WPA-PSK
 * allows the choice of either TKIP or AES. PSK also supports "auto" mode, in which the supplicant
 * and access point negotiate to chose the best encryption.
 *
 * The new Network Wireless Preference panel with WPA support also adds the ability to enter WEP
 * keys using a passphrase, similar to the PSK passphrase. However, while the driver supports the
 * PSK passphrase, the WEP passphrase is implemented by the panel, not the driver. WEP keys must 
 * always be send to the driver in fixed length binary format.
 *
 *
 *      Roaming
 *
 * The supplicant also implements a roaming algorithm. The user (through network preferences or 
 * the driver API, see below) sets up a list of "preferred" (ie. configured) access points, and
 * optionally designates one as "current." The supplicant will then try to connect first to the 
 * "current" access point, and if that fails, to any others it sees in the preferred list, based
 * on signal strength. Once connected, it will stay connected until the driver indicates a loss of
 * connection, at which time the supplicant will scan and attempt to connect to another preferred
 * access point, based on signal strength. If no connection can be made for 20 seconds, the 
 * supplicant will bring down the driver and the user must manually reconnect when back in range.
 * 
 *
 *      Preferred SSID List
 *
 * Because the supplicant must control the connection process in order to perform authentication
 * and roaming, it must have access to the settings of each configured access point. These 
 * configurations are maintained by the driver in a "preferred SSID list." Applications
 * wishing to configure connections (including the Network Preference Panel) should do so by
 * adding entries into the list (see below for legacy API behavior). Each entry in the list
 * is described by a PrefSSID_t structure, which contains all the SSID-specific settings for
 * connection to that access point. In addition, one and only one of the entries may be marked
 * as "current" which means it will be the first attempted connection the next time the driver
 * is loaded. 
 *
 * If a connection to an SSID is lost, the supplicant will perform a scan and attempts to connect 
 * to the best SSID that appears in both the scan list and the preferred list (unless roaming is 
 * disabled). All settings from the new SSID will be used except the IP-related settings (DHCP 
 * versus Static IP, Subnet, etc); these settings must remain constant. Thus the driver cannot roam 
 * across network boundries. When the driver is unloaded, the last connected SSID is automatically 
 * made "current" so that it will be tried first the next time the driver loads.
 *
 * The driver attempts to remain compatible with the old API (versions priors to 1.3). In this 
 * case, only one connection can be setup at a time using individual netIFSetting calls, and it
 * is always considered the "current" connection. When the driver connects, the individual 
 * preferences are added to the preferred SSID list and made current. Similarly, when an
 * application using the new API marks a preferred SSID entry "current," it's settings are 
 * copied to the legacy driver settings. The end result is that the legacy settings will always
 * reflect the "current" preferred SSID in the list, and updating either will update the other.
 *
 *
 *-----------------------------------------------------------------------------------------------------
 *
 *  PrefSSIDGet     PrefSSIDReq_t,  Read-only. Can be read at any time. Get the settings for a
 *                  PrefSSID_t      preferred SSID. The caller passes in a buffer of size
 *                                  PrefSSID_t, but containing a PrefSSIDReq_t (the PrefSSIDReq_t
 *                                  will be overwritten by the results). The PrefSSIDReq_t 
 *                                  contains the following fields:
 *
 *                                       match_method:  indicates how to find entry (by index or SSID)
 *                                       list_index:    index of SSID to get (for index matches)
 *                                       ssid_len:      length of SSID to get (for SSID matches)
 *                                       ssid:          SSID to get (for SSID matches)
 *                                  
 *                                  The caller may request an SSID either by specifying the SSID
 *                                  itself, or by specifying the index of the SSID in the list. The
 *                                  latter technique is only intended to be used for iterating through 
 *                                  the list, as indexes of specific SSIDs in the list may change 
 *                                  over time as entries are added or deleted. The search method is 
 *                                  specified by the match_method field, which can be either of the 
 *                                  following:
 *
 *                                      PREF_SSID_USE_SSID      find entry using the SSID
 *                                      PREF_SSID_USE_INDEX     find entry using the index
 *
 *                                  The SSID configuration is returned in a PrefSSID_t structure
 *                                  which contains the following fields:
 *
 *                                      flags:              special flags
 *                                      IP_addr:            static IP address
 *                                      netmask:            static netmask
 *	                                    ip_broadcast:       static IP broadcast
 *	                                    use_DHCP:           true to use DHCP       
 *	                                    manual_DNS:         true to use static DNS addresses with DHCP
 *                                      ad_hoc:             true if this is an ad-hoc network
 *                                      shared_key_authen:  true for shared-key authentication 
 *                                      channel:            channel number for ad-hoc connections     
 *                                      ssid_len:           length of SSID
 *                                      ssid:               SSID
 *	                                    security:           type of security to use
 *                                      encryption:         type of encryption to use
 *                                      WEPkeyLength:       WEP key type (40/64 or 104/128)
 *                                      WEPkeyIndex:        active WEP key index
 *                                      WEPkeys:            WEP keys (write only)
 *                                      WPAKey:             WPA key in binary form (write only)
 *                                      WPAPassphrase:      WPA key passphrase (write only)
 *
 *                                  Some of the fields require further explanation:
 *
 *                                  Flags: Two flags are defined. _SSID_FLAG_CURRENT indicates
 *                                  the current SSID and is read-only; it can only be set through
 *                                  netIFSetting80211PrefSSIDMakeCurrent. It is also set automatically
 *                                  in some circumstances (see description in section "Preferred
 *                                  SSID List" above). _SSID_FLAG_DISALLOW_ROAMING indicates that
 *                                  this SSID should not be used while roaming.
 *
 *                                  IP_addr, netmask, ip_broadcast, use_DHCP, and manual_DNS: these
 *                                  IP settings are only used during the initial connection. Once
 *                                  connected, a roam to a new access point ignores these settings.
 *                                  Also note that ip_broadcast should normally be set to 0 to use
 *                                  the default.
 *
 *                                  shared_key_authen: this is for backward compatibility and is
 *                                  used only with WEP security. It refers to the old 802.11 
 *                                  "authentication" that occurs before association, and is not
 *                                  related to WPA authentication. It should always be set false 
 *                                  unless the access point requires it to be true. In general,
 *                                  using "shared key authentication" is considered a security 
 *                                  risk.
 *
 *                                  ssid: this is a 32 character string that is not guaranteed to
 *                                  be NULL terminated. If the SSID is shorter than 32 characters,
 *                                  however, the remaining bytes should be zero-filled.
 *
 *                                  security, encryption: see netIFSetting80211SecurityMode and
 *                                  netIFSetting80211EncryptionType for more details.
 *
 *                                  WEPkeyLength: see netIFSetting80211WEPKeyLength.
 *
 *                                  WEPKeys, WPAKey, WPAPassphrase: these are write only. Only
 *                                  one of the three should be set, the others must be zeroed
 *                                  out. WEPKeys is an array of four keys, and the unused keys
 *                                  in the array should be zeroed out. Added in API version 1.3.
 *                    
 *  PrefSSIDSet          PrefSSID_t Write-only. Can be written at any time. Add a preferred SSID
 *                                  to the list or modify an existing preferred SSID entry. The
 *                                  caller must pass in all settings even if changing an existing
 *                                  entry. The entry is matched based on SSID. An SSID cannot be
 *                                  set "current" through this setting; use the setting 
 *                                  netIFSetting80211PrefSSIDMakeCurrent instead. See description
 *                                  of netIFSetting80211PrefSSIDGet for more details. Added in API
 *                                  version 1.3.
 *
 *  PrefSSIDMakeCurrent             Write-only. Can be written at any time. Make the specified
 *                    PrefSSIDReq_t SSID "current." Whenever the driver loads, it attempts to 
 *                                  connect to the current SSID. Setting an SSID current also
 *                                  copies its settings into the legacy driver preferences. When the
 *                                  driver connects, it copies the legacy settings into the SSID
 *                                  list and marks it current. When the driver disconnects, it makes 
 *                                  the last SSID current. Thus the "current" SSID will change
 *                                  as the device connects and disconnects from various SSIDs. In 
 *                                  general, using a mixture of legacy (pre-1.3) API and current API 
 *                                  to setup connections is not recommended, as it may be difficult to 
 *                                  predict which SSID is set current at connect time. Note that the 
 *                                  application should always set one of the SSIDs current, or else 
 *                                  the legacy settings will be used on the next connection. See 
 *                                  netIFSetting80211PrefSSIDGet for description of the PrefSSIDReq_t
 *                                  parameter. Added in API version 1.3.
 *
 *  PrefSSIDDelete    PrefSSIDReq_t Write-only. Can be written at any time. Delete an entry from the
 *                                  preferred SSID list. If the deleted entry is the current entry,
 *                                  the legacy settings are reset to their defaults (open, no SSID); 
 *                                  if no new SSID is made current, the supplicant will connect to
 *                                  the strongest open SSID it can find. See description of setting
 *                                  netIFSetting80211PrefSSIDGet for use of PrefSSIDReq_t. Added in 
 *                                  API version 1.3.  
 *
 *  PrefSSIDCount        UInt16     Read-only. Can be read at any time. Get the number of entries
 *                                  in the preferred SSID list. Used to allow callers to iterate
 *                                  through the list. Added in API version 1.3.
 *
 *----------------------------------------------------------------------------------------------------*/


// max length of SSID
#define MAX_SSID_LENGTH                         32

// Network types
#define _80211_NETWORK_TYPE_INFRASTRUCTURE      0       // Access Point
#define _80211_NETWORK_TYPE_AD_HOC              1       // Ad-Hoc (device-to-device)

// Authentication modes
#define _80211_AUTH_MODE_OPEN                   0       // always use this mode for WPA/WPA2
#define _80211_AUTH_MODE_SHARED_KEY             1       // old 802.11 shared-key authentication, generally not recommended

// WEP key length
#define _80211_WEP_KEY_LENGTH_40_BIT            0
#define _80211_WEP_KEY_LENGTH_64_BIT            0       // convenience, synonym for 40-bit
#define _80211_WEP_KEY_LENGTH_104_BIT           1
#define _80211_WEP_KEY_LENGTH_128_BIT           1       // convenience, synonym for 104-bit

// Encryption types
#define _80211_ENCRYPT_WEP                      0       // original WEP encryption only
#define _80211_ENCRYPT_TKIP                     1       // WPA TKIP encryption
#define _80211_ENCRYPT_AES                      2       // 802.11i AES/CCMP advanced encryption
#define _80211_ENCRYPT_AUTO_TKIP_AES            3       // Auto

// Security modes
#define _80211_SECURITY_NONE                    0       // open mode
#define _80211_SECURITY_WEP                     1       // WEP only
#define _80211_SECURITY_WPA_PSK                 2       // WPA Personal security (ie. WEP and WPA-PSK)
#define _80211_SECURITY_EAP                     3       // WPA Enterprise security (ie. EAP methods)

// max number of characters in a WPA-PSK security passphrases
#define MIN_WPA_PASSPHRASE_LENGTH               8       // for WPA-PSK
#define MAX_WPA_PASSPHRASE_LENGTH               63      // for WPA-PSK
#define WPA_KEY_LENGTH                          32      // for WPA-PSK

// flags for preferred SSIDs
#define _SSID_FLAG_CURRENT                      0x00000001  // read-only, set automatically or by PrefSSIDMakeCurrent
#define _SSID_FLAG_DISALLOW_ROAMING             0x00000002  // exclude this SSID from roaming list


/*====================================================================================
 * NetIFSettingGet/Set() public setting structures
 *====================================================================================*/

// format of a WEP key
#define MAX_WEP_KEY_LENGTH_BYTES                (26/2)  // 26 digits, 13 bytes
typedef struct _tagWEPKey_t
{
    UInt16         keyIndex;
    UInt16         keyLength;
    char           keyData[MAX_WEP_KEY_LENGTH_BYTES];
} WEPKey_t;

// format of packet statistics
typedef struct _tagPacketStats_t
{
    UInt32 TotalPackets;
    UInt32 UnicastPackets;
    UInt32 MulticastPackets;
    UInt32 BroadcastPackets;
    UInt32 TotalBytes;
    UInt32 UnicastBytes;
    UInt32 MulticastBytes;
    UInt32 BroadcastBytes;
} PacketStats_t;

// format for driver statistics
typedef struct _tagStats_t
{
    PacketStats_t GoodTx;
    PacketStats_t GoodRx;
    PacketStats_t BadTx;
    PacketStats_t BadRx;
} Stats_t;

// format of a single AP descriptor in a scan
typedef struct _tagAP_ScanResult_t
{
    UInt8  MacAddress[6];                   // BSSID of Access Point or IBSS
    UInt8  pad1[2];
    UInt32 SSIDLength;                      // SSID length (SSID not NULL-terminated)
    UInt8  SSID[MAX_SSID_LENGTH];           // SSID
    UInt8  BSSType;                         // network type (BSS or IBSS)
    UInt8  pad2[3];
    UInt32 RSSI;                            // signal strength, interpret as Int32
    UInt16 BeaconPeriod;
    UInt8  DTIMPeriod;
    UInt8  pad3;
    UInt16 ATIMWindow;
    UInt16 BasicRateMask;
    UInt16 Capability;                      // indicates secured or open
    UInt8  PhyParams[7];
    UInt8  WMEInfoElement[9];
    UInt8  WMEParams[26];
    UInt8  WPALength;
    UInt8  WPAInfoElement[30];
    UInt8  RSNLength;
    UInt8  RSNInfoElement[44];
} AP_ScanResult_t;

// format of the AP list from a scan
#define BSS_TOTAL_DESCRIPTORS             12
typedef struct _tagAP_ScanInfo_t
{
    UInt32          NumScanResults;         // caller must zero this field out!
    AP_ScanResult_t ScanResult[BSS_TOTAL_DESCRIPTORS];
} AP_ScanInfo_t;


// format of a preferred SSID request (for PrefSSIDGet, PrefSSIDDelete, PrefSSIDMakeCurrent)
#define PREF_SSID_USE_SSID                 0    // select preferred SSID by matching SSID 
#define PREF_SSID_USE_INDEX                1    // select preferred SSID by index in list
typedef struct _tagPrefSSIDReq_t
{
    UInt16          match_method;               // PREF_SSID_USE_SSID or PREF_SSID_USE_INDEX
    UInt16          list_index;                 // if match_method = PREF_SSID_USE_INDEX
    UInt16          ssid_len;                   // if match_method = PREF_SSID_USE_SSID
    char            ssid[MAX_SSID_LENGTH];      // if match_method = PREF_SSID_USE_SSID
} PrefSSIDReq_t;

// format of a preferred SSID configuration (for PrefSSIDSet, returned by PrefSSIDGet)
typedef struct _tagPrefSSID_t
{
    // properties of this configuration
    UInt32          flags;                      // define various properties (see defines)
    
	// IP settings
	UInt32			IP_addr;                    // static or requested (DHCP) IP
	UInt32        	netmask;                    // subnet mask
	UInt32          ip_broadcast;               // IP broadcast address
	Boolean        	use_DHCP;                   // use DHCP or static IP
	Boolean         manual_DNS;                 // true = ignore DNS from DHCP
	
	// WLAN configuration
    Boolean			ad_hoc;                     // true = ad-hoc mode, false = infrustructure mode
    Boolean		 	shared_key_authen;          // true = shared key authentication, false = open authentication
    UInt16		    channel;                    // desired ad-hoc channel
    UInt16          ssid_len;                   // SSID length (SSID not NULL-terminated)
    char            ssid[MAX_SSID_LENGTH];      // desired SSID

	// security settings
	UInt16          security;                   // security mode
    UInt16          encryption;                 // encryption type
    UInt16 			WEPkeyLength;               // 40-bit or 128-bit WEP keys
    UInt16  		WEPkeyIndex;                // active WEP key index
    UInt8           WEPkeys[4][MAX_WEP_KEY_LENGTH_BYTES];           // WEP keys -- binary form
    UInt8           WPAKey[WPA_KEY_LENGTH];                         // WPA-PSK binary key
    char            WPAPassphrase[MAX_WPA_PASSPHRASE_LENGTH+1];     // WPA-PSK passphrase
} PrefSSID_t;


/*====================================================================================
 * NetIFSettingGet/Set() public settings
 *====================================================================================*/
typedef enum
{
    // placeholders for Symbol S24 compatibility settings (deprecated, use Palm OS or Janus API)
    _netIFSettingS24Device = netIFSettingCustom,// Is this an S24 device? (read only)
    _netIFSettingS24EssID,                      // Adapter's ESS ID
    _netIFSettingS24AccessPointBSSID,           // AP's BSSID (read only)
    _netIFSettingS24DriverVersion,              // Driver version (read only)
    _netIFSettingS24AssociationStatus,          // Association status (read only)
    _netIFSettingS24MKKCallsign,                // MKK callsign (read only)
    _netIFSettingS24CountryText,                // Country text (read only)
    _netIFSettingS24FirmwareVersion,            // Firmware version (read only)
    _netIFSettingS24FirmwareDate,               // Firmware date (read only)
    _netIFSettingS24Preferences,                // S24 Preferences
    
    // Janam API (standard Palm OS 802.11 netIFsettings also supported)
    netIFSetting80211ReqESSID,                  // Get/set the preferred ESSID 
    netIFSetting80211RSSI,                      // Get signal RSSI 
    netIFSetting80211NetworkType,               // Get/set network type (ad-hoc or infrustructure)
    netIFSetting80211TxRate,                    // Get TX rate of current connection (in Mbits/sec)   
    netIFSetting80211Channel,                   // Get current (I)BSS channel
    netIFSetting80211ReqAdHocChannel,           // Get/set channel for Ad Hoc network creation
    netIFSetting80211MACAddress,                // Get the MAC address
    netIFSetting80211ReqMACAddr,                // Get/set MAC address in preferences (overrides EEPROM)
    netIFSetting80211AuthenticationMode,        // Get/set authentication mode (open or shared key)
    netIFSetting80211Statistics,                // Get the driver statistics
    netIFSetting80211Scan,                      // Perform a scan and return resulting AP list
    netIFSetting80211StartBackgroundScan,       // Start a background scan and return
    netIFSetting80211ScanInProgress,            // Check status of background scan
    netIFSetting80211ScanResults,               // Get the AP list from the last scan
    netIFSetting80211ScanMode,                  // Flag to open driver in scan mode (no association)
    netIFSetting80211HighestSecurityMode,       // Get highest supported security mode (WEP, PSK, or EAP)
    netIFSetting80211SecurityMode,              // Set the security mode (none, WEP, PSK, EAP)
    netIFSetting80211HighestEncryptionType,     // Get highest supported encryption type (WEP, TKIP, or AES)
    netIFSetting80211EncryptionType,            // Set the encryption type (WEP, TKIP, AES)
    netIFSetting80211WEPKeyIndex,               // Set current WEP key index
    netIFSetting80211WEPKeyLength,              // Set current WEP key length (40/64 or 128 bit)
    netIFSetting80211WEPKey,                    // Set WEP key
    netIFSetting80211CancelScan,                // Cancel the current background scan
    netIFSetting80211ManualDNS,                 // true if DNS specified by user (ie. not through DHCP)
    netIFSetting80211Roaming,                   // Roaming control
    netIFSetting80211APIVersion,                // Get the API version
    netIFSetting80211Passphrase,                // Set security passphrase for WPA
    netIFSetting80211PrefSSIDGet,               // get the configuration info of a preferred SSID
    netIFSetting80211PrefSSIDSet,               // set the configuration info for a preferred SSID
    netIFSetting80211PrefSSIDMakeCurrent,       // make a preferred SSID the current SSID (using SSID)
    netIFSetting80211PrefSSIDDelete,            // delete a preferred SSID
    netIFSetting80211PrefSSIDCount,             // get number of preferred SSIDs
} wlanNetIfPublicSettings; 


/*----------------------------------------------------------------------------------------------------*
 *                   BSD license for Security Supplicant wpa_supplicant
 *----------------------------------------------------------------------------------------------------*
 *  WPA Supplicant
 *  ==============
 *
 *  Copyright (c) 2003-2006, Jouni Malinen <jkmaline@cc.hut.fi> and
 *  contributors
 *  All Rights Reserved.
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  3. Neither the name(s) of the above-listed copyright holder(s) nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *----------------------------------------------------------------------------------------------------*/


#endif
